/* (C) Copyright 1984,85,86,87 Walter L. Peacock   All Rights Reserved    */
/*  C K E R R O R . C             */
/*----------------------------------------------------------------------------
 * ckerror() is used to trap errors.  You should modify ckerror() to
 * conform to your error-processing requirements.   The current version uses the
 * ckerror.msg file which contains error messages.  The parameter 'where', if
 * starting with ASCII number, uses this as a line number in ckerror.msg.
 * The line from ckerror.msg is then displayed, followed by any further text in
 * 'where'.  If retcode is negative, the error is considered fatal and the
 * exit() function is called to return to the operating system.
 *---------------------------------------------------------------------------*/

#include "cbtree.h"   /* defines NOMEM */
#include "btfio.h"

#if CI | DC
#else
#include <ctype.h>
#endif

#if AmigaDOS & LC
#define printf iprintf
#endif

char *errmsg[] =
{
                     /* offset 0 NOT TO BE USED ! */
   "need memory",       /* errcode =    CK_NOMEM:   "module: filename"   */
   "btrinit",          /* errcode =    CK_BTRIN:    "module: indexname" */
   "file open",          /* errcode =   CK_OPEN:   "module: filename"  */
   "seek",             /* errcode =    CK_SEEK:   "module: filename"   */
   "read",             /* errcode =    CK_READ:   "module: filename"   */
   "write",             /* errcode =   CK_WRIT: "module: filename"   */
   "cbtree",             /* errcode =    CK_CBTR:   "module: indexname" */
   "stackop",          /* errcode =    CK_STAK:   "module: variable"   */
   "pushdown",          /* errcode =   CK_PUSH: "module: variable"   */
   "index maint"   /* errcode =  CK_INDX:    "module: variable"    */
};

void ckerror(errcode, err_s) /* error output routine */
int errcode;
char *err_s;
{
   extern int atoi(), bt_open(), strlen();
   extern void exit();
   char *msg_fatal = "FATAL ";
#if MSDOS
   char *cntrl_s = "%serror: %s error in %s %s\r\n";
#else
   char *cntrl_s = "%serror: %s error in %s %s\n";
#endif
   int fd, fdmsg;
   char *fdgets();
   int fatal;
   int errnum;   /* leading error number which MAY be imbedded in err_s */
   char msgline[83];   /* place to read in error messages */
   char outline[83];   /* place to store output line */

   errnum = atoi(err_s);                  /* grab leading number (if any) */

   *msgline = NUL;
   if ((fdmsg = bt_open("ckerror.msg", O_RDONLY)) == ERR)    /* ascii read */
   {
      printf("\nCan't open ckerror.msg");
   }
   else    /* open ok */
   {
      while (errnum-- != 0)   /* find desired message */
         if (fdgets(msgline, 82, fdmsg) == NULL)   /* read a line */
         {
            *msgline = NUL;
            printf("\nmessage # is beyond end of ckerror.msg.\n");
            break;
         }
      close(fdmsg);
   }

   if (errcode < 0)    /* if negative, this is a FATAL error ! */
   {
      errcode = -errcode - 1; /* adjust to fit array bounds */
      fatal = 1;
   }
   else                /* non-fatal: write error out and continue */
   {
      fatal = 0;
      --errcode;                   /* adjust to fit array bounds */
   }


   if ((fd = bt_open("error.fil", O_CREAT|O_RDWR, S_IRDWR)) == ERR)
   {
      printf("\nCan't open error.fil");
   }
   else
   {
      lseek(fd, 0L, 2);                   /* move to end-of-file */
      puts("");
      printf(cntrl_s, fatal ? msg_fatal : "",
            errmsg[errcode], msgline, err_s);

      sprintf(outline, cntrl_s, fatal ? msg_fatal : "",
            errmsg[errcode], msgline, err_s);
      write(fd, outline, strlen(outline));
      close(fd);
   }

   if (fatal)
      exit(ERR);
}

/* this table is also used by userbtr.c */
BTOPTYPE optypes[] =
{
   {   "DELETE"   ,  DELETE  },   /* SORTED Alpa order */
   {   "DELTKY"   ,  DELTKY  },
   {   "GETALL"   ,  GETALL  },
   {   "GETFRST" ,   GETFRST },
   {   "GETGE"   ,   GETGE   },
   {   "GETGT"   ,   GETGT   },
   {   "GETKEYS" ,   GETKEYS },
   {   "GETLAST" ,   GETLAST },
   {   "GETLE"   ,   GETLE   },
   {   "GETLT"   ,   GETLT   },
   {   "GETNXT"   ,  GETNXT  },
   {   "GETPAR"   ,  GETPAR  },
   {   "GETPRV"   ,  GETPRV  },
   {   "GETREC"   ,  GETREC  },
   {   "GETSEQ"   ,  GETSEQ  },
   {   "INSERT"   ,  INSERT  },
   {   "ISRTKY"   ,  ISRTKY  },
   {   "NEWLOC"   ,  NEWLOC  },
   {   "BAD OPTYPE",  98     },
   {   "EXIT"     ,   99     }
};
int noptypes = sizeof(optypes)/sizeof(*optypes);  /* # optypes in array */

char *cberrmsg[] =
{
               /* offest 0 NOT TO BE USED ! */
   "BTERROR",    /* "module", &btcommo     */
   "BTNOTFND",   /* "module", &btcommo     */
   "BTSTRUCT",   /* "module", &btcommo     */
   "NOMEM",      /* "module", &btcommo     */
   "BTCPYBRO",   /* "module", &btcommo     */
   "BTLOCK",     /* "module", &btcommo     */
   "BTDUPKEY",   /* "module", &btcommo     */
   "",           /* -8 is unused           */
   "BTABORT"     /* "module", &btcommo     */
};

void cberror(errcode, err_s, btcp) /* CBTREE error output routine */
int errcode;
char *err_s;
BTC *btcp;    /* btcommo pointer */
{
   extern int bt_open();
   extern BTOPTYPE optypes[];
   extern int noptypes;
   int fd;
   int typeoff;       /* offset into optype table */
#if MSDOS
   char *ctrl_s = "%s error in %s: %s, key = '%s', loc = %lu\r\n";
#else
   char *ctrl_s = "%s error in %s: %s, key = '%s', loc = %lu\n";
#endif
   char outline[83];

   for (typeoff = 0; typeoff < noptypes - 2; ++typeoff)
      if (btcp->btoptype == optypes[typeoff].op_value)
         break;

   errcode = -errcode - 1;   /* make into a positive index */

   puts("");
   printf(ctrl_s, optypes[typeoff].op_name, err_s, cberrmsg[errcode],
         btcp->btkey, btcp->btloc);
   if (errcode != -(BTNOTFND+1))   /* don't record BTNOTFND errors */
   {
      if ((fd = bt_open("error.fil", O_CREAT|O_RDWR, S_IRDWR)) == ERR)
      {
         printf("can't open error.fil");
      }
      else
      {
         lseek(fd, 0L, 2);                   /* move to end-of-file */
         sprintf(outline, ctrl_s, optypes[typeoff].op_name, err_s,
               cberrmsg[errcode], btcp->btkey, btcp->btloc);
         write(fd, outline, strlen(outline));
         close(fd);
      }
   }
}
